@@ -336,10 +336,12 @@ void ndDetectionThread::ProcessPacket(ndDetectionQueueEntry *entry)
         );
 
         // XXX: Preserve app_protocol.
-        ndEF->detected_protocol = nd_ndpi_proto_find(
-            ndpi_rc.master_protocol,
-            ndEF
-        );
+        if (ndEF->flags.soft_dissector.load() == false) {
+            ndEF->detected_protocol = nd_ndpi_proto_find(
+                ndpi_rc.master_protocol,
+                ndEF
+            );
+        }
 
         if (ndEF->detected_protocol == ND_PROTO_UNKNOWN) {
             ndEF->detected_protocol = nd_ndpi_proto_find(
@@ -442,14 +444,14 @@ void ndDetectionThread::ProcessPacket(ndDetectionQueueEntry *entry)
             }
         }
 
-        CopyHostname(
+        nd_set_hostname(
             ndEF->host_server_name, ndEFNF->host_server_name,
             min((size_t)ND_FLOW_HOSTNAME, (size_t)sizeof(ndEFNF->host_server_name))
         );
 
         // Determine application based on master protocol metadata for
         // Protocol / Application "Twins"
-        switch (ndEF->master_protocol()) {
+        switch (ndEF->detected_protocol) {
 
         case ND_PROTO_SPOTIFY:
             SetDetectedApplication(entry, nd_apps->Lookup("netify.spotify"));
@@ -536,9 +538,7 @@ void ndDetectionThread::ProcessPacket(ndDetectionQueueEntry *entry)
         }
 
         // Additional protocol-specific processing...
-        nd_proto_id_t nd_proto = ndEF->master_protocol();
-
-        switch (nd_proto) {
+        switch (ndEF->master_protocol()) {
 
         case ND_PROTO_TLS:
         case ND_PROTO_QUIC:
@@ -551,7 +551,7 @@ void ndDetectionThread::ProcessPacket(ndDetectionQueueEntry *entry)
 
             if (ndEF->ssl.server_cn[0] == '\0' &&
                 ndEFNFP.tls_quic.serverCN != NULL) {
-                CopyHostname(
+                nd_set_hostname(
                     ndEF->ssl.server_cn,
                     ndEFNFP.tls_quic.serverCN,
                     ND_FLOW_TLS_CNLEN
@@ -864,7 +864,7 @@ void ndDetectionThread::ProcessPacket(ndDetectionQueueEntry *entry)
 
             if (ndEF->ssl.server_cn[0] == '\0' &&
                 ndEFNFP.tls_quic.serverCN != NULL) {
-                CopyHostname(
+                nd_set_hostname(
                     ndEF->ssl.server_cn,
                     ndEFNFP.tls_quic.serverCN,
                     ND_FLOW_TLS_CNLEN
@@ -932,15 +932,29 @@ void ndDetectionThread::ProcessPacket(ndDetectionQueueEntry *entry)
             }
         break;
         case ND_PROTO_DNS:
-        case ND_PROTO_MDNS:
-        case ND_PROTO_LLMNR:
             //nd_dprintf("DNS flow updated.\n");
             break;
+        case ND_PROTO_MDNS:
+            if (ndEF->has_mdns_domain_name()) {
+                flow_update = true;
+                ndEF->flags.detection_updated = true;
+            }
+            //nd_dprintf("mDNS flow updated: %s, %s, %s\n",
+            //    ndEF->host_server_name, ndEFNF->host_server_name, ndEF->mdns.domain_name);
+            break;
+        case ND_PROTO_LLMNR:
+            //nd_dprintf("LLMNR flow updated.\n");
+            break;
         default:
             break;
         }
     }
-
+#if 0
+    if (ndEF->detected_protocol == ND_PROTO_MDNS) {
+        nd_dprintf("mDNS flow updated: packets: %lu, check extra packets: %s.\n",
+            ndEF->detection_packets.load(), (check_extra_packets) ? "yes" : "no");
+    }
+#endif
     // Flow detection complete.
     if (ndEF->flags.detection_init.load() && ! check_extra_packets)
         ndEF->flags.detection_complete = true;
@@ -1114,27 +1128,4 @@ void ndDetectionThread::SetDetectedApplication(ndDetectionQueueEntry *entry, nd_
     );
 }
 
-void ndDetectionThread::CopyHostname(char *dst, const char *src, size_t length)
-{
-    ssize_t i;
-
-    // Sanitize host server name; RFC 952 plus underscore for SSDP.
-    for (i = 0; i < (ssize_t)length; i++) {
-
-        if (isalnum(src[i]) || src[i] == '-' ||
-            src[i] == '_' || src[i] == '.')
-            dst[i] = tolower(src[i]);
-        else {
-            dst[i] = '\0';
-            break;
-        }
-    }
-
-    // Ensure dst is terminated.
-    dst[length - 1] = '\0';
-
-    // Right-trim dots.
-    for (--i; i > -1 && dst[i] == '.'; i--) dst[i] = '\0';
-}
-
 // vi: expandtab shiftwidth=4 softtabstop=4 tabstop=4
